package com.yeejoin.equipmanage.controller;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.yeejoin.amos.boot.biz.common.bo.ReginParams;
import com.yeejoin.equipmanage.common.utils.CommonResponseUtil;
import com.yeejoin.equipmanage.common.utils.DateUtils;
import com.yeejoin.equipmanage.common.vo.IotDataVO;
import com.yeejoin.equipmanage.config.PersonIdentify;
import com.yeejoin.equipmanage.service.IEmergencyService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.util.ObjectUtils;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.typroject.tyboot.core.foundation.enumeration.UserType;
import org.typroject.tyboot.core.foundation.utils.DateTimeUtil;
import org.typroject.tyboot.core.restful.doc.TycloudOperation;
import org.typroject.tyboot.core.restful.utils.ResponseModel;

import java.math.BigDecimal;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.LocalDate;
import java.time.ZoneId;
import java.util.*;

import static org.typroject.tyboot.core.foundation.utils.DateTimeUtil.ISO8601_DATE_HOUR_MIN_SEC;

/**
 *
 * 四横八纵应急模块接口
 *
 *
 */
@RestController
@Api(tags = "四横八纵应急模块Api")
@RequestMapping(value = "/emergency", produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
public class EmergencyController extends AbstractBaseController {

    @Autowired
	IEmergencyService iEmergencyService;

	/**
	 * 通过bizOrgCode过滤
	 */
	@TycloudOperation(ApiLevel = UserType.AGENCY)
	@ApiOperation("系统状态")
	@GetMapping(value = "/system/state")
	public ResponseModel getSystemState() {
        ReginParams reginParams = getSelectedOrgInfo();
        String bizOrgCode = reginParams.getPersonIdentity().getBizOrgCode();
		return CommonResponseUtil.success(iEmergencyService.getSystemState(bizOrgCode));
	}

	/**
	 * CAFS-消防水箱信息
	 */
	@TycloudOperation(ApiLevel = UserType.AGENCY)
	@ApiOperation("CAFS-消防水箱信息")
	@GetMapping(value = "/cafs-tank")
	public ResponseModel getCAFSWaterTankInfo() {
		ReginParams reginParams = getSelectedOrgInfo();
		String bizOrgCode = reginParams.getPersonIdentity().getBizOrgCode();
		return CommonResponseUtil.success(iEmergencyService.getCAFSWaterTankInfo());
	}

	@TycloudOperation(ApiLevel = UserType.AGENCY)
	@ApiOperation("应急物资")
	@GetMapping(value = "/emergencyMaterials")
	public Map<String, Object> emergencyMaterials()  {
		return iEmergencyService.emergencyMaterials();
	}

	@TycloudOperation(ApiLevel = UserType.AGENCY)
	@ApiOperation("消防系统/消防车")
	@GetMapping(value = "/systemAndCar")
	public Map<String, Object> systemAndCar()  {
		return iEmergencyService.systemAndCar();
	}

	@TycloudOperation(ApiLevel = UserType.AGENCY)
	@ApiOperation("应急物资-详情")
	@GetMapping(value = "/emergencyMaterialsDetails")
	public Page<Map<String, Object>> emergencyMaterialsDetails(@RequestParam(value = "bizOrgCode", required = false) String bizOrgCode ,
															   @RequestParam(value = "pageNumber") int pageNumber,
															   @RequestParam(value = "pageSize") int pageSize)  {
		Page<Map<String, Object>> page = new Page<>(pageNumber, pageSize);
		if (ObjectUtils.isEmpty(bizOrgCode)){
			ReginParams reginParams = getSelectedOrgInfo();
			bizOrgCode = reginParams.getPersonIdentity().getBizOrgCode();
		}
		return iEmergencyService.emergencyMaterialsDetails(page, bizOrgCode);
	}

	@TycloudOperation(ApiLevel = UserType.AGENCY)
	@ApiOperation("消防炮列表")
	@GetMapping(value = "/fireMonitor")
	public Page<Map<String, Object>> selectFireMonitor(@RequestParam(value = "bizOrgCode", required = false) String bizOrgCode,
													   @RequestParam(value = "pageNumber") int pageNumber,
													   @RequestParam(value = "pageSize") int pageSize,
													   @RequestParam(value = "code") String code)  {
		Page<Map<String, Object>> page = new Page<>(pageNumber, pageSize);
		if (ObjectUtils.isEmpty(bizOrgCode)){
			ReginParams reginParams = getSelectedOrgInfo();
			bizOrgCode = reginParams.getPersonIdentity().getBizOrgCode();
		}
		return iEmergencyService.selectFireMonitor(page, bizOrgCode, code);
	}

	@TycloudOperation(ApiLevel = UserType.AGENCY)
	@ApiOperation("泡沫罐列表")
	@GetMapping(value = "/selectFoamTank")
	public Page<Map<String, Object>> selectFoamTank(@RequestParam(value = "pageNumber") int pageNumber,
													@RequestParam(value = "pageSize") int pageSize
													)  {
		Page<Map<String, Object>> page = new Page<>(pageNumber, pageSize);
		return iEmergencyService.selectFoamTank(page);
	}

	@TycloudOperation(ApiLevel = UserType.AGENCY)
	@ApiOperation("排油系统列表")
	@GetMapping(value = "/selectOilDrainage")
	public Page<Map<String, Object>> selectOilDrainage(@RequestParam(value = "bizOrgCode", required = false) String bizOrgCode ,
													   @RequestParam(value = "pageNumber") int pageNumber,
													   @RequestParam(value = "pageSize") int pageSize,
													   @RequestParam(value = "code") String code)  {
		Page<Map<String, Object>> page = new Page<>(pageNumber, pageSize);
		if (ObjectUtils.isEmpty(bizOrgCode)){
			ReginParams reginParams = getSelectedOrgInfo();
			bizOrgCode = reginParams.getPersonIdentity().getBizOrgCode();
		}
		return iEmergencyService.selectOilDrainage(page, bizOrgCode, code);
	}

	@TycloudOperation(ApiLevel = UserType.AGENCY)
	@ApiOperation("压力流量")
	@GetMapping(value = "/selectPressureFlow")
	public Page<Map<String, Object>> selectPressureFlow(@RequestParam(value = "bizOrgCode", required = false) String bizOrgCode ,
													   @RequestParam(value = "pageNumber") int pageNumber,
													   @RequestParam(value = "pageSize") int pageSize)  {
		Page<Map<String, Object>> page = new Page<>(pageNumber, pageSize);
		if (ObjectUtils.isEmpty(bizOrgCode)){
			ReginParams reginParams = getSelectedOrgInfo();
			bizOrgCode = reginParams.getPersonIdentity().getBizOrgCode();
		}
		return iEmergencyService.selectPressureFlow(page, bizOrgCode);
	}

	@TycloudOperation(ApiLevel = UserType.AGENCY)
	@ApiOperation("火灾自动报警系统告警分析-折线图")
	@GetMapping(value = "/selectAlarmAnalysis")
	public List<Map<String, Object>> selectPressureFlow(@RequestParam(value = "startDate") String startDate,
														@RequestParam(value = "endDate") String endDate,
														@RequestParam(value = "type") String type,
														@RequestParam(value = "systemType") String systemType)throws Exception   {

		ArrayList<Map<String, Object>> result = new ArrayList<>();
		List<Map<String, Object>> maps = iEmergencyService.selectAlarmAnalysis(startDate, endDate, "0", "", systemType);

		if ("week".equals(type)) {
			String beginMonth = startDate.substring(5,7);
			String endMonth = endDate.substring(5,7);
			List<Map<String, String>> weeksMapInterval = DateUtils.getWeeksMapInterval(DateUtils.dateToStringMonth(startDate));
			for (Map<String, String> week : weeksMapInterval) {
				Date weekStart = null;
				Date weekEnd = null;
				try {
					weekStart = DateUtils.dateParse(week.get("weekStart"), DateUtils.DATE_PATTERN);
					weekEnd = DateUtils.dateParse(week.get("weekEnd"), DateUtils.DATE_PATTERN);
				} catch (ParseException e) {
					e.printStackTrace();
				}
				long startTime = weekStart.getTime();
				long endTime = weekEnd.getTime();
				HashMap<String, Object> mapData = new HashMap<>();
				for (Map<String, Object> map : maps) {
					Date check = null;
					try {
						check = DateUtils.dateParse(String.valueOf(map.get("date")), DateUtils.DATE_PATTERN);
					} catch (ParseException e) {
						e.printStackTrace();
					}
					if (!ObjectUtils.isEmpty(check) && check.getTime() >= startTime && check.getTime() <= endTime) {
						if (!ObjectUtils.isEmpty(mapData)) {
							BigDecimal old = new BigDecimal(String.valueOf(mapData.get("faultNum")));
							BigDecimal add = new BigDecimal(String.valueOf(map.get("faultNum")));
							BigDecimal now = old.add(add);
							mapData.put("faultNum", String.valueOf(now));

							BigDecimal firePumpsOld = new BigDecimal(String.valueOf(mapData.get("alarmNum")));
							BigDecimal firePumpsAdd = new BigDecimal(String.valueOf(map.get("alarmNum")));
							BigDecimal firePumpsNow = firePumpsOld.add(firePumpsAdd);
							mapData.put("alarmNum", String.valueOf(firePumpsNow));

							BigDecimal stabilizedOld = new BigDecimal(String.valueOf(mapData.get("shieldNum")));
							BigDecimal stabilizedAdd = new BigDecimal(String.valueOf(map.get("shieldNum")));
							BigDecimal stabilizedNow = stabilizedOld.add(stabilizedAdd);
							mapData.put("shieldNum", String.valueOf(stabilizedNow));
						} else {
							mapData.put("name", week.get("name"));
							mapData.put("faultNum", map.get("faultNum"));
							mapData.put("alarmNum", map.get("alarmNum"));
							mapData.put("shieldNum", map.get("shieldNum"));
						}

					}
				}
				result.add(mapData);
			}
			if(!StringUtils.isEmpty(beginMonth) && !beginMonth.equals(endMonth)){
				weeksMapInterval = DateUtils.getWeeksMapInterval(DateUtils.dateToStringMonth(endDate));
				for (Map<String, String> weeks : weeksMapInterval) {
					Date weekStart1 = null;
					Date weekEnd1 = null;
					try {
						weekStart1 = DateUtils.dateParse(weeks.get("weekStart"), DateUtils.DATE_PATTERN);
						weekEnd1 = DateUtils.dateParse(weeks.get("weekEnd"), DateUtils.DATE_PATTERN);
					} catch (ParseException e) {
						e.printStackTrace();
					}
					long startTime1 = weekStart1.getTime();
					long endTime1 = weekEnd1.getTime();
					HashMap<String, Object> mapData1 = new HashMap<>();
					for (Map<String, Object> map : maps) {
						Date check = null;
						try {
							check = DateUtils.dateParse(String.valueOf(map.get("date")), DateUtils.DATE_PATTERN);
						} catch (ParseException e) {
							e.printStackTrace();
						}
						if (!ObjectUtils.isEmpty(check) && check.getTime() >= startTime1 && check.getTime() <= endTime1) {
							if (!ObjectUtils.isEmpty(mapData1)) {
								BigDecimal old = new BigDecimal(String.valueOf(mapData1.get("faultNum")));
								BigDecimal add = new BigDecimal(String.valueOf(map.get("faultNum")));
								BigDecimal now = old.add(add);
								mapData1.put("faultNum", String.valueOf(now));

								BigDecimal firePumpsOld = new BigDecimal(String.valueOf(mapData1.get("alarmNum")));
								BigDecimal firePumpsAdd = new BigDecimal(String.valueOf(map.get("alarmNum")));
								BigDecimal firePumpsNow = firePumpsOld.add(firePumpsAdd);
								mapData1.put("alarmNum", String.valueOf(firePumpsNow));

								BigDecimal stabilizedOld = new BigDecimal(String.valueOf(mapData1.get("shieldNum")));
								BigDecimal stabilizedAdd = new BigDecimal(String.valueOf(map.get("shieldNum")));
								BigDecimal stabilizedNow = stabilizedOld.add(stabilizedAdd);
								mapData1.put("shieldNum", String.valueOf(stabilizedNow));
							} else {
								mapData1.put("name", weeks.get("name"));
								mapData1.put("faultNum", map.get("faultNum"));
								mapData1.put("alarmNum", map.get("alarmNum"));
								mapData1.put("shieldNum", map.get("shieldNum"));
							}

						}
					}
					result.add(mapData1);
				}
			}

		} else if ("day".equals(type)) {
			return maps;
		} else if ("month".equals(type)) {
			String date = startDate.substring(0, 4) + "-01" + "-01";
			return iEmergencyService.selectAlarmAnalysis(startDate, endDate, "1", date, systemType);
		}

		return result;
	}

	@TycloudOperation(ApiLevel = UserType.AGENCY)
	@ApiOperation("火灾自动报警系统告警分析-饼图")
	@GetMapping(value = "/selectAlarmAnalysisCount")
	public List<Map<String, Object>> selectAlarmAnalysisCount(@RequestParam(value = "startDate") String startDate,
															  @RequestParam(value = "endDate") String endDate,
															  @RequestParam(value = "systemType") String systemType) {
		return iEmergencyService.selectAlarmAnalysisCount(systemType, startDate, endDate);
	}

	@TycloudOperation(ApiLevel = UserType.AGENCY)
	@ApiOperation("消防炮灭火系统告警分析-饼图")
	@GetMapping(value = "/fireCannonAlarmAnalysisPie")
	public List<Map<String, Object>> fireCannonAlarmAnalysisPie(@RequestParam(value = "startDate") String startDate,
																@RequestParam(value = "endDate") String endDate,
																@RequestParam(value = "systemType") String systemType) {

		return iEmergencyService.selectFireCannonAlarmAnalysisPie(startDate, endDate, systemType);
	}

	@TycloudOperation(ApiLevel = UserType.AGENCY)
	@ApiOperation("消防给水系统告警分析-饼图")
	@GetMapping(value = "/waterSystemAlarmAnalysisPie")
	public List<Map<String, Object>> waterSystemAlarmAnalysisPie(@RequestParam(value = "startDate") String startDate,
																 @RequestParam(value = "endDate") String endDate,
																 @RequestParam(value = "systemType") String systemType) {
		return iEmergencyService.waterSystemAlarmAnalysisPie(startDate, endDate, systemType);
	}

	@TycloudOperation(ApiLevel = UserType.AGENCY)
	@ApiOperation("消防给水系统告警分析-小图")
	@GetMapping(value = "/waterSystemAlarmAnalysis")
	public List<Map<String, Object>> waterSystemAlarmAnalysis(@RequestParam(value = "startDate") String startDate,
															  @RequestParam(value = "endDate") String endDate,
															  @RequestParam(value = "type") String type,
															  @RequestParam(value = "systemType") String systemType) {

		ArrayList<Map<String, Object>> result = new ArrayList<>();
		List<Map<String, Object>> maps = iEmergencyService.waterSystemAlarmAnalysis(startDate, endDate, "0", "", systemType);
		if ("week".equals(type)) {
			String beginMonth = startDate.substring(5,7);
			String endMonth = endDate.substring(5,7);
			List<Map<String, String>> weeksMapInterval = DateUtils.getWeeksMapInterval(DateUtils.dateToStringMonth(startDate));
			for (Map<String, String> week : weeksMapInterval) {
				Date weekStart = null;
				Date weekEnd = null;
				try {
					weekStart = DateUtils.dateParse(week.get("weekStart"), DateUtils.DATE_PATTERN);
					weekEnd = DateUtils.dateParse(week.get("weekEnd"), DateUtils.DATE_PATTERN);
				} catch (ParseException e) {
					e.printStackTrace();
				}
				long startTime = weekStart.getTime();
				long endTime = weekEnd.getTime();
				HashMap<String, Object> mapData = new HashMap<>();
				for (Map<String, Object> map : maps) {
					Date check = null;
					try {
						check = DateUtils.dateParse(String.valueOf(map.get("date")), DateUtils.DATE_PATTERN);
					} catch (ParseException e) {
						e.printStackTrace();
					}
					if (!ObjectUtils.isEmpty(check) && check.getTime() >= startTime && check.getTime() <= endTime) {
						if (!ObjectUtils.isEmpty(mapData)) {
							BigDecimal old = new BigDecimal(String.valueOf(mapData.get("controlCabinetAlarmNum")));
							BigDecimal add = new BigDecimal(String.valueOf(map.get("controlCabinetAlarmNum")));
							BigDecimal now = old.add(add);
							mapData.put("controlCabinetAlarmNum", String.valueOf(now));

							BigDecimal firePumpsOld = new BigDecimal(String.valueOf(mapData.get("firePumpsAlarmNum")));
							BigDecimal firePumpsAdd = new BigDecimal(String.valueOf(map.get("firePumpsAlarmNum")));
							BigDecimal firePumpsNow = firePumpsOld.add(firePumpsAdd);
							mapData.put("firePumpsAlarmNum", String.valueOf(firePumpsNow));

							BigDecimal stabilizedOld = new BigDecimal(String.valueOf(mapData.get("stabilizedPumpAlarmNum")));
							BigDecimal stabilizedAdd = new BigDecimal(String.valueOf(map.get("stabilizedPumpAlarmNum")));
							BigDecimal stabilizedNow = stabilizedOld.add(stabilizedAdd);
							mapData.put("stabilizedPumpAlarmNum", String.valueOf(stabilizedNow));
						} else {
							mapData.put("name", week.get("name"));
							mapData.put("controlCabinetAlarmNum", map.get("controlCabinetAlarmNum"));
							mapData.put("firePumpsAlarmNum", map.get("firePumpsAlarmNum"));
							mapData.put("stabilizedPumpAlarmNum", map.get("stabilizedPumpAlarmNum"));
						}

					}

				}
				result.add(mapData);
			}
			if(!StringUtils.isEmpty(beginMonth) && !beginMonth.equals(endMonth)){
				weeksMapInterval = DateUtils.getWeeksMapInterval(DateUtils.dateToStringMonth(endDate));
				for (Map<String, String> weeks : weeksMapInterval) {
					Date weekStart1 = null;
					Date weekEnd1 = null;
					try {
						weekStart1 = DateUtils.dateParse(weeks.get("weekStart"), DateUtils.DATE_PATTERN);
						weekEnd1 = DateUtils.dateParse(weeks.get("weekEnd"), DateUtils.DATE_PATTERN);
					} catch (ParseException e) {
						e.printStackTrace();
					}
					long startTime1 = weekStart1.getTime();
					long endTime1 = weekEnd1.getTime();
					HashMap<String, Object> mapData1 = new HashMap<>();
					for (Map<String, Object> map : maps) {
						Date check = null;
						try {
							check = DateUtils.dateParse(String.valueOf(map.get("date")), DateUtils.DATE_PATTERN);
						} catch (ParseException e) {
							e.printStackTrace();
						}
						if (!ObjectUtils.isEmpty(check) && check.getTime() >= startTime1 && check.getTime() <= endTime1) {
							if (!ObjectUtils.isEmpty(mapData1)) {
								BigDecimal old = new BigDecimal(String.valueOf(mapData1.get("controlCabinetAlarmNum")));
								BigDecimal add = new BigDecimal(String.valueOf(map.get("controlCabinetAlarmNum")));
								BigDecimal now = old.add(add);
								mapData1.put("controlCabinetAlarmNum", String.valueOf(now));

								BigDecimal firePumpsOld = new BigDecimal(String.valueOf(mapData1.get("firePumpsAlarmNum")));
								BigDecimal firePumpsAdd = new BigDecimal(String.valueOf(map.get("firePumpsAlarmNum")));
								BigDecimal firePumpsNow = firePumpsOld.add(firePumpsAdd);
								mapData1.put("firePumpsAlarmNum", String.valueOf(firePumpsNow));

								BigDecimal stabilizedOld = new BigDecimal(String.valueOf(mapData1.get("stabilizedPumpAlarmNum")));
								BigDecimal stabilizedAdd = new BigDecimal(String.valueOf(map.get("stabilizedPumpAlarmNum")));
								BigDecimal stabilizedNow = stabilizedOld.add(stabilizedAdd);
								mapData1.put("stabilizedPumpAlarmNum", String.valueOf(stabilizedNow));
							} else {
								mapData1.put("name", weeks.get("name"));
								mapData1.put("controlCabinetAlarmNum", map.get("controlCabinetAlarmNum"));
								mapData1.put("firePumpsAlarmNum", map.get("firePumpsAlarmNum"));
								mapData1.put("stabilizedPumpAlarmNum", map.get("stabilizedPumpAlarmNum"));
							}

						}
					}
					result.add(mapData1);
				}
			}

		} else if ("day".equals(type)) {
			return maps;
		} else if ("month".equals(type)) {
			String date = startDate.substring(0, 4) + "-01" + "-01";
			return iEmergencyService.waterSystemAlarmAnalysis(startDate, endDate, "1", date, systemType);
		}

		return result;
	}


	@TycloudOperation(ApiLevel = UserType.AGENCY)
	@ApiOperation("消防炮系统告警分析-折线图")
	@GetMapping(value = "/fireCannonSystemAlarmAnalysis")
	public List<Map<String, Object>> fireCannonSystemAlarmAnalysis(@RequestParam(value = "startDate") String startDate,
																   @RequestParam(value = "endDate") String endDate,
																   @RequestParam(value = "type") String type,
																   @RequestParam(value = "systemType") String systemType) {

		ArrayList<Map<String, Object>> result = new ArrayList<>();
		List<Map<String, Object>> maps = iEmergencyService.fireCannonSystemAlarmAnalysis(startDate, endDate, "0", "", systemType);
		if ("week".equals(type)) {
			String beginMonth = startDate.substring(5,7);
			String endMonth = endDate.substring(5,7);
			List<Map<String, String>> weeksMapInterval = DateUtils.getWeeksMapInterval(DateUtils.dateToStringMonth(startDate));
			for (Map<String, String> week : weeksMapInterval) {
				Date weekStart = null;
				Date weekEnd = null;
				try {
					weekStart = DateUtils.dateParse(week.get("weekStart"), DateUtils.DATE_PATTERN);
					weekEnd = DateUtils.dateParse(week.get("weekEnd"), DateUtils.DATE_PATTERN);
				} catch (ParseException e) {
					e.printStackTrace();
				}
				long startTime = weekStart.getTime();
				long endTime = weekEnd.getTime();
				HashMap<String, Object> mapData = new HashMap<>();
				for (Map<String, Object> map : maps) {
					Date check = null;
					try {
						check = DateUtils.dateParse(String.valueOf(map.get("date")), DateUtils.DATE_PATTERN);
					} catch (ParseException e) {
						e.printStackTrace();
					}
					if (!ObjectUtils.isEmpty(check) && check.getTime() >= startTime && check.getTime() <= endTime) {
						if (!ObjectUtils.isEmpty(mapData)) {
							BigDecimal old = new BigDecimal(String.valueOf(mapData.get("powerLossNum")));
							BigDecimal add = new BigDecimal(String.valueOf(map.get("powerLossNum")));
							BigDecimal now = old.add(add);
							mapData.put("powerLossNum", String.valueOf(now));

							BigDecimal firePumpsOld = new BigDecimal(String.valueOf(mapData.get("faultNum")));
							BigDecimal firePumpsAdd = new BigDecimal(String.valueOf(map.get("faultNum")));
							BigDecimal firePumpsNow = firePumpsOld.add(firePumpsAdd);
							mapData.put("faultNum", String.valueOf(firePumpsNow));

							BigDecimal stabilizedOld = new BigDecimal(String.valueOf(mapData.get("fireAlarmNum")));
							BigDecimal stabilizedAdd = new BigDecimal(String.valueOf(map.get("fireAlarmNum")));
							BigDecimal stabilizedNow = stabilizedOld.add(stabilizedAdd);
							mapData.put("fireAlarmNum", String.valueOf(stabilizedNow));

							BigDecimal shieldOld = new BigDecimal(String.valueOf(mapData.get("shieldNum")));
							BigDecimal shieldAdd = new BigDecimal(String.valueOf(map.get("shieldNum")));
							BigDecimal shieldNow = shieldOld.add(shieldAdd);
							mapData.put("shieldNum", String.valueOf(shieldNow));
						} else {
							mapData.put("name", week.get("name"));
							mapData.put("shieldNum", map.get("shieldNum"));
							mapData.put("fireAlarmNum", map.get("fireAlarmNum"));
							mapData.put("faultNum", map.get("faultNum"));
							mapData.put("powerLossNum", map.get("powerLossNum"));
						}

					}

				}
				result.add(mapData);
			}
			if(!StringUtils.isEmpty(beginMonth) && !beginMonth.equals(endMonth)){
				weeksMapInterval = DateUtils.getWeeksMapInterval(DateUtils.dateToStringMonth(endDate));
				for (Map<String, String> weeks : weeksMapInterval) {
					Date weekStart1 = null;
					Date weekEnd1 = null;
					try {
						weekStart1 = DateUtils.dateParse(weeks.get("weekStart"), DateUtils.DATE_PATTERN);
						weekEnd1 = DateUtils.dateParse(weeks.get("weekEnd"), DateUtils.DATE_PATTERN);
					} catch (ParseException e) {
						e.printStackTrace();
					}
					long startTime1 = weekStart1.getTime();
					long endTime1 = weekEnd1.getTime();
					HashMap<String, Object> mapData1 = new HashMap<>();
					for (Map<String, Object> map : maps) {
						Date check = null;
						try {
							check = DateUtils.dateParse(String.valueOf(map.get("date")), DateUtils.DATE_PATTERN);
						} catch (ParseException e) {
							e.printStackTrace();
						}
						if (!ObjectUtils.isEmpty(check) && check.getTime() >= startTime1 && check.getTime() <= endTime1) {
							if (!ObjectUtils.isEmpty(mapData1)) {
								BigDecimal old = new BigDecimal(String.valueOf(mapData1.get("powerLossNum")));
								BigDecimal add = new BigDecimal(String.valueOf(map.get("powerLossNum")));
								BigDecimal now = old.add(add);
								mapData1.put("powerLossNum", String.valueOf(now));

								BigDecimal firePumpsOld = new BigDecimal(String.valueOf(mapData1.get("faultNum")));
								BigDecimal firePumpsAdd = new BigDecimal(String.valueOf(map.get("faultNum")));
								BigDecimal firePumpsNow = firePumpsOld.add(firePumpsAdd);
								mapData1.put("faultNum", String.valueOf(firePumpsNow));

								BigDecimal stabilizedOld = new BigDecimal(String.valueOf(mapData1.get("fireAlarmNum")));
								BigDecimal stabilizedAdd = new BigDecimal(String.valueOf(map.get("fireAlarmNum")));
								BigDecimal stabilizedNow = stabilizedOld.add(stabilizedAdd);
								mapData1.put("fireAlarmNum", String.valueOf(stabilizedNow));

								BigDecimal shieldOld = new BigDecimal(String.valueOf(mapData1.get("shieldNum")));
								BigDecimal shieldAdd = new BigDecimal(String.valueOf(map.get("shieldNum")));
								BigDecimal shieldNow = shieldOld.add(shieldAdd);
								mapData1.put("shieldNum", String.valueOf(shieldNow));
							} else {
								mapData1.put("name", weeks.get("name"));
								mapData1.put("powerLossNum", map.get("powerLossNum"));
								mapData1.put("faultNum", map.get("faultNum"));
								mapData1.put("fireAlarmNum", map.get("fireAlarmNum"));
								mapData1.put("shieldNum", map.get("shieldNum"));
							}

						}
					}
					result.add(mapData1);
				}
			}


		} else if ("day".equals(type)) {
			return maps;
		} else if ("month".equals(type)) {
			String date = startDate.substring(0, 4) + "-01" + "-01";
			return iEmergencyService.fireCannonSystemAlarmAnalysis(startDate, endDate, "1", date, systemType);
		}

		return result;
	}

	@PersonIdentify
	@TycloudOperation(ApiLevel = UserType.AGENCY)
	@GetMapping(value = "/getEquipExpiryList")
	@ApiOperation(httpMethod = "GET", value = "四横八纵-临期设备列表", notes = "四横八纵-临期设备列表")
	public ResponseModel getEquipExpiryList(@ApiParam("临期天数-近多少天") @RequestParam(value = "expiryDayNum", required = false) Integer expiryDayNum) {
		expiryDayNum = Objects.isNull(expiryDayNum) ? 180 : expiryDayNum;
		return CommonResponseUtil.success(iEmergencyService.getEquipExpiryList(expiryDayNum));
	}

	@PersonIdentify
	@TycloudOperation(ApiLevel = UserType.AGENCY)
	@GetMapping(value = "/getEquipExpiryStatistics")
	@ApiOperation(httpMethod = "GET", value = "四横八纵-临期设备统计", notes = "四横八纵-临期设备统计")
	public ResponseModel getEquipExpiryStatistics(@ApiParam("临期天数-近多少天") @RequestParam(value = "expiryDayNum", required = false) Integer expiryDayNum) {
		expiryDayNum = Objects.isNull(expiryDayNum) ? 180 : expiryDayNum;
		return CommonResponseUtil.success(iEmergencyService.getEquipExpiryStatistics(expiryDayNum));
	}

	@PersonIdentify
	@TycloudOperation(ApiLevel = UserType.AGENCY)
	@GetMapping(value = "/getStockEquipStatistics")
	@ApiOperation(httpMethod = "GET", value = "四横八纵-报废设备统计", notes = "四横八纵-报废设备统计")
	public ResponseModel getStockEquipStatistics() {
		return CommonResponseUtil.success(iEmergencyService.getStockEquipStatistics());
	}

}
